@@ -58,30 +58,15 @@ export class GraphHopper {
         return this._route;
     }
 
-    constructor({query}) {
+    constructor({query, route}) {
         this._session = new Soup.Session({ user_agent : 'gnome-maps/' + pkg.version });
         this._key     = "VCIHrHj0pDKb8INLpT4s5hVadNmJ1Q3vi0J4nJYP";
         this._baseURL = "https://graphhopper.com/api/1/route?";
         this._locale  = GLib.get_language_names()[0];
-        this._route   = new Route();
-        this.storedRoute = null;
+        this._route   = route;
         this._query = query;
     }
 
-    _updateFromStored() {
-        GLib.idle_add(null, () => {
-            if (!this.storedRoute)
-                return;
-
-            this.route.update({ path: this.storedRoute.path,
-                                turnPoints: this.storedRoute.turnPoints,
-                                distance: this.storedRoute.distance,
-                                time: this.storedRoute.time,
-                                bbox: this.storedRoute.bbox });
-            this.storedRoute = null;
-        });
-    }
-
     _queryGraphHopper(points, transportationType, callback) {
         let url = this._buildURL(points, transportationType);
         let msg = Soup.Message.new('GET', url);
@@ -105,11 +90,6 @@ export class GraphHopper {
     }
 
     fetchRoute(points, transportationType) {
-        if (this.storedRoute) {
-            this._updateFromStored();
-            return;
-        }
-
         this._queryGraphHopper(points, transportationType,
                                (result, exception) => {
             if (exception) {
