@@ -32,13 +32,11 @@ import Shumate from 'gi://Shumate';
 
 import {Application} from './application.js';
 import {ExportViewDialog} from './exportViewDialog.js';
-import {FavoritesPopover} from './favoritesPopover.js';
 import * as Geoclue from './geoclue.js';
 import * as GeocodeFactory from './geocode.js';
 import {HeaderBarLeft, HeaderBarRight} from './headerBar.js';
 import {MapView} from './mapView.js';
 import {PlaceBar} from './placeBar.js';
-import {PlaceStore} from './placeStore.js';
 import {PrintOperation} from './printOperation.js';
 import {SearchBar} from './searchBar.js';
 import * as Service from './service.js';
@@ -106,14 +104,20 @@ export class MainWindow extends Adw.ApplicationWindow {
             hexpand: true,
             vexpand: true });
 
+        /* couldn't get the CSS styling to apply to the licemse widget
+         * when defining it in the .ui defintion
+         */
+        this._license = new Shumate.License({ halign: Gtk.Align.END,
+                                              valign: Gtk.Align.START });
+        this._licenseRevealer.child = this._license;
+
+        // init license for the initial map source
+        this._license.append_map_source(this._mapView.mapSource);
+        this._revealLicense();
         this._mapOverlay.child = this._mapView;
 
         this._mapOverlay.add_overlay(
-            new ZoomAndRotateControls({ mapView: this._mapView,
-                                        halign:  Gtk.Align.START,
-                                        valign:  Gtk.Align.START,
-                                        margin_start: 6,
-                                        margin_top: 6 }));
+            new ZoomAndRotateControls({ mapView: this._mapView }));
 
         this._mapView.gotoUserLocation(false);
 
@@ -165,10 +169,10 @@ export class MainWindow extends Adw.ApplicationWindow {
         });
 
         let popover = searchBar.popover;
-        popover.connect('selected', () => this._mapView.grab_focus());
+        popover.connect('selected', () => this._mapView.map.grab_focus());
 
         this._buttonPressGesture = new Gtk.GestureSingle();
-        this._mapView.add_controller(this._buttonPressGesture);
+        this._mapView.map.add_controller(this._buttonPressGesture);
         this._buttonPressGesture.connect('begin', () => popover.popdown());
         return searchBar;
     }
@@ -203,6 +207,34 @@ export class MainWindow extends Adw.ApplicationWindow {
         });
     }
 
+    _revealLicense() {
+        this._licenseRevealer.reveal_child = true;
+
+        if (this._licenseRevealerTimeoutId)
+            GLib.source_remove(this._licenseRevealerTimeoutId);
+
+        this._licenseRevealerTimeoutId = GLib.timeout_add(null, 10000, () => {
+            this._licenseRevealer.reveal_child = false;
+            this._licenseRevealerTimeoutId = 0;
+
+            return false;
+        });
+    }
+
+    _setMapType(mapType) {
+        const mapSource = this._mapView.mapSource;
+
+        // remove previous map source from license
+        if (mapSource)
+            this._license.remove_map_source(mapSource);
+
+        this._mapView.setMapType(mapType);
+
+        // set license for new map source
+        this._license.append_map_source(this._mapView.mapSource);
+        this._revealLicense();
+    }
+
     _initActions() {
         let actions = {
             'about': {
@@ -298,7 +330,7 @@ export class MainWindow extends Adw.ApplicationWindow {
                 onChangeState: (action, variant) => {
                     action.set_state(variant);
                     const state = variant.get_boolean();
-                    this._mapView.setMapType(state ? MapView.MapType.VECTOR : MapView.MapType.STREET);
+                    this._setMapType(state ? MapView.MapType.VECTOR : MapView.MapType.STREET);
                 }
             };
         }
@@ -311,6 +343,7 @@ export class MainWindow extends Adw.ApplicationWindow {
         this.connect('notify::default-width', () => this._onSizeChanged());
         this.connect('notify::default-height', () => this._onSizeChanged());
         this.connect('notify::maximized', () => this._onMaximizedChanged());
+        this.connect('notify::is-active', () => this._onIsActiveChanged());
 
         let viewport = this._mapView.map.viewport;
 
@@ -324,6 +357,28 @@ export class MainWindow extends Adw.ApplicationWindow {
         this._updateZoomButtonsSensitivity();
     }
 
+    _onIsActiveChanged() {
+        if (this._inactiveId) {
+            GLib.source_remove(this._inactiveId);
+            this._inactiveId = 0;
+        }
+
+        /* reveal the license when the window gets active if it has
+         * been inactive for at least 10 s inbetween
+         */
+        if (this.is_active) {
+            if (this._isIdle)
+                this._revealLicense();
+            this._isIdle = false;
+        } else {
+            this._inactiveId = GLib.timeout_add(null, 10000, () => {
+                this._isIdle = true;
+                this._inactiveId = 0;
+                return false;
+            });
+        }
+    }
+
     _updateZoomButtonsSensitivity() {
         let zoomLevel = this._mapView.map.viewport.zoom_level;
         let maxZoomLevel = this._mapView.map.viewport.max_zoom_level;
@@ -357,7 +412,6 @@ export class MainWindow extends Adw.ApplicationWindow {
 
         this._searchBar = this._createSearchBar();
         this._headerBar.title_widget = this._searchBar;
-        this._searchBar.placeEntry.grab_focus();
 
         Application.geoclue.connect('notify::state',
                                     this._updateLocationSensitivity.bind(this));
@@ -552,13 +606,13 @@ export class MainWindow extends Adw.ApplicationWindow {
     }
 
     _onStreetViewActivate() {
-        this._mapView.setMapType(MapView.MapType.STREET);
+        this._setMapType(MapView.MapType.STREET);
     }
 
     _onAerialViewActivate() {
         // don't attempt to switch to aerial if we don't have tiles for it
         if (Service.getService().tiles.aerial) {
-            this._mapView.setMapType(MapView.MapType.AERIAL);
+            this._setMapType(MapView.MapType.AERIAL);
         }
     }
 
@@ -570,6 +624,8 @@ export class MainWindow extends Adw.ApplicationWindow {
 
         if (reveal)
             this._splitView.sidebar.focusStartEntry();
+        else
+            this._mapView.map.grab_focus();
     }
 
     _setRevealSidebar(value) {
@@ -593,7 +649,7 @@ export class MainWindow extends Adw.ApplicationWindow {
             copyright: _("Copyright © 2011 – 2023 Red Hat, Inc. and The GNOME Maps authors"),
             license_type: Gtk.License.GPL_2_0,
             version: pkg.version,
-            website: 'https://apps.gnome.org/app/org.gnome.Maps',
+            website: 'https://apps.gnome.org/Maps/',
             issue_url: 'https://gitlab.gnome.org/GNOME/gnome-maps/-/issues/new',
             transient_for: this
         });
@@ -697,5 +753,6 @@ GObject.registerClass({
                         'overlay',
                         'mapOverlay',
                         'splitView',
-                        'breakpoint' ]
+                        'breakpoint',
+                        'licenseRevealer']
 }, MainWindow);
