@@ -19,20 +19,10 @@
  * Author: Damián Nohales <damiannohales@gmail.com>
  */
 
-import Gio from 'gi://Gio';
-import GLib from 'gi://GLib';
 import GObject from 'gi://GObject';
-import Graphene from 'gi://Graphene';
 import Gtk from 'gi://Gtk';
 
-import {Application} from './application.js';
-import * as GeocodeFactory from './geocode.js';
-import {Place} from './place.js';
 import {PlaceView} from './placeView.js';
-import {PlaceButtons} from './placeButtons.js';
-import {PlaceFormatter} from './placeFormatter.js';
-import {PlaceStore} from './placeStore.js';
-import * as Utils from './utils.js';
 
 /* Maximum width of the popover content before it's forced to wrap */
 const MAX_CONTENT_WIDTH = 350;
@@ -42,13 +32,7 @@ const HEIGHT_MARGIN = 100;
 
 export class MapBubble extends Gtk.Popover {
 
-    constructor(params) {
-        let place = params.place;
-        delete params.place;
-
-        let mapView = params.mapView;
-        delete params.mapView;
-
+    constructor({place, mapView, ...params}) {
         super(params);
 
         let content = new PlaceView({ place, mapView, visible: true });
@@ -60,6 +44,11 @@ export class MapBubble extends Gtk.Popover {
                                                            child: content });
         this.child = scrolledWindow;
 
+        /* focus on the map when the bubble is closed, to allow continuing
+         * keyboard navigation
+         */
+        this.connect('closed', () => mapView.map.grab_focus());
+
         this.get_style_context().add_class("map-bubble");
     }
 }
