@@ -17,12 +17,15 @@
  * Author: Jonas Danielsson <jonas@threetimestwo.org>
  */
 
+import Adw from 'gi://Adw';
+import GLib from 'gi://GLib';
 import Gio from 'gi://Gio';
-import Gdk from 'gi://Gdk';
 import Shumate from 'gi://Shumate';
+import GnomeMaps from 'gi://GnomeMaps';
 
 import * as Service from './service.js';
 import * as Utils from './utils.js';
+import { generateMapStyle } from './mapStyle/mapStyle.js';
 
 /* Converts a tile URI format from Champlain style to Shumate style.
  * e.g. from
@@ -67,28 +70,24 @@ export function createPrintSource() {
 }
 
 export function createVectorSource() {
-    const [_status, styleFile] = Gio.file_new_for_uri('resource://org/gnome/Maps/styles/osm-liberty/style.json').load_contents(null);
-    const style = Utils.getBufferText(styleFile);
+    const start = GLib.get_monotonic_time();
+    const style = generateMapStyle({
+        colorScheme: Adw.StyleManager.get_default().dark ? "dark" : "light",
+        language: Utils.getLanguage(),
+        textScale: Adw.LengthUnit.to_px(Adw.LengthUnit.SP, 1, null),
+    });
+    const end = GLib.get_monotonic_time();
+    Utils.debug(`Map style generated in ${(end - start) / 1000} ms.`);
 
-    const source = Shumate.VectorRenderer.new("vector-tiles", style);
+    const source = Shumate.VectorRenderer.new("vector-tiles", JSON.stringify(style));
     source.set_license("© OpenMapTiles © OpenStreetMap contributors");
     source.set_license_uri("https://www.openstreetmap.org/copyright");
 
     const sprites = Shumate.VectorSpriteSheet.new();
-    const [_status2, spritesJsonFile] = Gio.file_new_for_uri('resource://org/gnome/Maps/styles/osm-liberty/sprites.json').load_contents(null);
-    const spritesJson = Utils.getBufferText(spritesJsonFile);
-    sprites.add_page(
-        Gdk.Texture.new_from_resource('/org/gnome/Maps/styles/osm-liberty/sprites.png'),
-        spritesJson,
-        1
-    );
-    const [_status3, sprites2xJsonFile] = Gio.file_new_for_uri('resource://org/gnome/Maps/styles/osm-liberty/sprites@2x.json').load_contents(null);
-    const sprites2xJson = Utils.getBufferText(sprites2xJsonFile);
-    sprites.add_page(
-        Gdk.Texture.new_from_resource('/org/gnome/Maps/styles/osm-liberty/sprites@2x.png'),
-        sprites2xJson,
-        2
-    );
+    const spriteSource = new GnomeMaps.SpriteSource({"color-scheme": "light"});
+    const [_status4, shieldsJsonFile] = Gio.file_new_for_uri('resource://org/gnome/Maps/shields/shields.json').load_contents(null);
+    spriteSource.load_shield_defs(Utils.getBufferText(shieldsJsonFile));
+    spriteSource.set_fallback(sprites);
     source.set_sprite_sheet(sprites);
 
     return source;
