@@ -67,7 +67,8 @@ const MapMaxZoom = 19;
 const OUTLINE_LUMINANCE_THREASHHOLD = 0.9;
 
 // color used for turn-by-turn-based routes (non-transit)
-const TURN_BY_TURN_ROUTE_COLOR = '0000FF';
+const TURN_BY_TURN_ROUTE_COLOR = '62a0ea';
+const TURN_BY_TURN_ROUTE_OUTLINE_COLOR = '1a5fb4';
 
 // line width for route lines
 const ROUTE_LINE_WIDTH = 5;
@@ -141,7 +142,6 @@ export class MapView extends Gtk.Overlay {
 
         this.child = this.map;
 
-        this._initLicense();
         this.setMapType(mapType ?? this._getStoredMapType());
 
         this._initScale();
@@ -194,9 +194,17 @@ export class MapView extends Gtk.Overlay {
         this._contextMenu = new Gtk.PopoverMenu({ menu_model: menuModel, has_arrow: false });
         this._contextMenu.set_parent(this);
 
-        this._clickGesture = new Gtk.GestureClick({ button: Gdk.BUTTON_SECONDARY });
-        this._clickGesture.connect('pressed', this._onClickGesturePressed.bind(this));
-        this.map.add_controller(this._clickGesture);
+        this._primaryClickGesture =
+            new Gtk.GestureClick({ button: Gdk.BUTTON_PRIMARY });
+        this._primaryClickGesture.connect('pressed',
+                                          this._onPrimaryClick.bind(this));
+        this.map.add_controller(this._primaryClickGesture);
+
+        this._secondaryClickGesture =
+            new Gtk.GestureClick({ button: Gdk.BUTTON_SECONDARY });
+        this._secondaryClickGesture.connect('pressed',
+                                            this._onSecondaryClick.bind(this));
+        this.map.add_controller(this._secondaryClickGesture);
 
         this._longPressGesture = new Gtk.GestureLongPress({ touch_only: true });;
         this._longPressGesture.connect('pressed',
@@ -246,10 +254,14 @@ export class MapView extends Gtk.Overlay {
     _initScale() {
         let showScale = Application.settings.get('show-scale');
 
-        this._scale = new Shumate.Scale({ visible:  showScale,
-                                          viewport: this.map.viewport,
-                                          halign:   Gtk.Align.START,
-                                          valign:   Gtk.Align.END });
+        this._scale = new Shumate.Scale({ visible:       showScale,
+                                          viewport:      this.map.viewport,
+                                          halign:        Gtk.Align.START,
+                                          valign:        Gtk.Align.END,
+                                          margin_start:  6,
+                                          margin_end:    6,
+                                          margin_top:    6,
+                                          margin_bottom: 6 });
 
         if (Utils.getMeasurementSystem() === Utils.METRIC_SYSTEM)
             this._scale.unit = Shumate.Unit.METRIC;
@@ -259,12 +271,6 @@ export class MapView extends Gtk.Overlay {
         this.add_overlay(this._scale);
     }
 
-    _initLicense() {
-        this._license = new Shumate.License({ halign: Gtk.Align.END,
-                                              valign: Gtk.Align.END });
-        this.add_overlay(this._license);
-    }
-
     _initMap() {
         let map = new Shumate.Map();
 
@@ -287,7 +293,7 @@ export class MapView extends Gtk.Overlay {
     }
 
     /* create and store a route layer, pass true to get a dashed line */
-    _createRouteLayer(dashed, lineColor, outlineColor, width) {
+    _createRouteLayer(dashed, lineColor, outlineColor, width, outlineWidth = 1) {
         let strokeColor = new Gdk.RGBA({ red:    Color.parseColor(lineColor, 0),
                                          green:  Color.parseColor(lineColor, 1),
                                          blue:   Color.parseColor(lineColor, 2),
@@ -307,11 +313,11 @@ export class MapView extends Gtk.Overlay {
                                alpha: 1.0 });
 
             routeLayer.outline_color = outlineStrokeColor;
-            routeLayer.outline_width = 1.0;
+            routeLayer.outline_width = outlineWidth;
         }
 
         this._routeLayers.push(routeLayer);
-        this.map.insert_layer_behind(routeLayer, this._instructionMarkerLayer);
+        this.map.insert_layer_behind(routeLayer, this._userLocationLayer);
 
         return routeLayer;
     }
@@ -354,7 +360,7 @@ export class MapView extends Gtk.Overlay {
     }
 
     _connectRouteSignals() {
-        let route = Application.routingDelegator.graphHopper.route;
+        let route = Application.routingDelegator.route;
         let transitPlan = Application.routingDelegator.transitRouter.plan;
         let query = Application.routeQuery;
 
@@ -421,12 +427,40 @@ export class MapView extends Gtk.Overlay {
         return this._mapType;
     }
 
-    setMapType(mapType) {
-        if (this._mapType && this._mapType === mapType)
+    _listenForVectorChanges() {
+        if (this._stopListeningForVectorChanges)
+            return;
+
+        const styleManager = Adw.StyleManager.get_default();
+        const settings = this.get_settings();
+
+        const darkNotify = styleManager.connect('notify::dark', () => {
+            this.setMapType(this._mapType, true);
+        });
+
+        const textSizeNotify = settings.connect('notify::gtk-xft-dpi', () => {
+            this.setMapType(this._mapType, true);
+        });
+
+        this._stopListeningForVectorChanges = () => {
+            styleManager.disconnect(darkNotify);
+            settings.disconnect(textSizeNotify);
+            this._stopListeningForVectorChanges = null;
+        }
+    }
+
+    setMapType(mapType, forceReload = false) {
+        if (this._mapType && this._mapType === mapType && !forceReload)
             return;
 
         this._mapType = mapType;
 
+        if (mapType !== MapView.MapType.VECTOR) {
+            if (this._stopListeningForVectorChanges) {
+                this._stopListeningForVectorChanges();
+            }
+        }
+
         let mapSource;
 
         if (mapType !== MapView.MapType.LOCAL) {
@@ -434,8 +468,10 @@ export class MapView extends Gtk.Overlay {
 
             if (mapType === MapView.MapType.AERIAL && tiles.aerial)
                 mapSource = MapSource.createAerialSource();
-            else if (mapType === MapView.MapType.VECTOR && Shumate.VectorRenderer.is_supported())
+            else if (mapType === MapView.MapType.VECTOR && Shumate.VectorRenderer.is_supported()) {
                 mapSource = MapSource.createVectorSource();
+                this._listenForVectorChanges();
+            }
             else
                 mapSource = MapSource.createStreetSource();
 
@@ -474,10 +510,8 @@ export class MapView extends Gtk.Overlay {
         }
 
         this._mapLayer = mapLayer;
+        mapLayer.connect("symbol-clicked", this._onSymbolClicked.bind(this));
 
-        if (this._mapSource)
-            this._license.remove_map_source(this._mapSource);
-        this._license.append_map_source(mapSource);
         this.map.viewport.set_reference_map_source(mapSource);
 
         this._mapSource = mapSource;
@@ -485,6 +519,102 @@ export class MapView extends Gtk.Overlay {
         this.emit("map-type-changed", mapType);
     }
 
+    _onSymbolClicked(_mapLayer, symbol) {
+        let placeType = GeocodeGlib.PlaceType.UNKNOWN;
+
+        const layerName = symbol.get_source_layer();
+        const featureId = symbol.get_feature_id();
+        const className = symbol.get_tag("class");
+
+        const featureType = featureId % 10;
+        const osmId = Math.floor(featureId / 10);
+
+        let osmType;
+        if (featureType === 0) {
+            osmType = GeocodeGlib.PlaceOsmType.NODE;
+        } else if (featureType === 1) {
+            osmType = GeocodeGlib.PlaceOsmType.WAY;
+        } else if (featureType === 4) {
+            osmType = GeocodeGlib.PlaceOsmType.RELATION;
+        } else {
+            osmType = GeocodeGlib.PlaceOsmType.UNKNOWN;
+        }
+
+        switch (layerName) {
+            case "place":
+                placeType = {
+                    "continent": GeocodeGlib.PlaceType.CONTINENT,
+                    "country": GeocodeGlib.PlaceType.COUNTRY,
+                    "state": GeocodeGlib.PlaceType.STATE,
+                    "province": GeocodeGlib.PlaceType.STATE,
+                    "city": GeocodeGlib.PlaceType.TOWN,
+                    "town": GeocodeGlib.PlaceType.TOWN,
+                    "village": GeocodeGlib.PlaceType.TOWN,
+                    "hamlet": GeocodeGlib.PlaceType.TOWN,
+                    "suburb": GeocodeGlib.PlaceType.SUBURB,
+                    "quarter": GeocodeGlib.PlaceType.SUBURB,
+                    "borough": GeocodeGlib.PlaceType.SUBURB,
+                    "island": GeocodeGlib.PlaceType.ISLAND,
+                    "neighbourhood": GeocodeGlib.PlaceType.ESTATE,
+                    "isolated_dwelling": GeocodeGlib.PlaceType.ESTATE,
+                }[className] ?? GeocodeGlib.PlaceType.UNKNOWN;
+                break;
+
+            case "water_name":
+                placeType = {
+                    "ocean": GeocodeGlib.PlaceType.OCEAN,
+                    "sea": GeocodeGlib.PlaceType.SEA,
+                }[className] ?? GeocodeGlib.PlaceType.DRAINAGE;
+                break;
+
+            case "poi":
+                placeType = GeocodeGlib.PlaceType.POINT_OF_INTEREST;
+                break;
+
+            case "mountain_peak":
+                placeType = GeocodeGlib.PlaceType.LAND_FEATURE;
+                break;
+
+            case "aerodrome_label":
+                placeType = GeocodeGlib.PlaceType.AIRPORT;
+                break;
+
+            case "aeroway":
+                placeType =  GeocodeGlib.PlaceType.POINT_OF_INTEREST;
+                break;
+
+            default:
+                return;
+        }
+
+        const place = new Place({
+            location: new Location({
+                latitude: symbol.latitude,
+                longitude: symbol.longitude
+            }),
+            placeType,
+            name: symbol.get_tag("name"),
+            osmId,
+            osmType,
+        });
+
+        const osmTags = {};
+        for (const key of symbol.get_keys()) {
+            if (key.startsWith("osm:")) {
+                osmTags[key.slice("osm:".length)] = symbol.get_tag(key);
+            } else if (key === "name" || key.startsWith("name:")) {
+                osmTags[key] = symbol.get_tag(key);
+            }
+        }
+        const mainTag = symbol.get_tag("tag");
+        if (mainTag) {
+            osmTags[mainTag] = symbol.get_tag("subtag") ?? symbol.get_tag("subclass");
+        }
+        place.osmTags = osmTags;
+
+        this.showPlace(place, false, true);
+    }
+
     _onShowScaleChanged() {
         this._scale.visible = Application.settings.get('show-scale');
     }
@@ -770,7 +900,7 @@ export class MapView extends Gtk.Overlay {
         let place = new Place({
             location: new Location({ latitude  : lat,
                                               longitude : lon }),
-            bounding_box: new GeocodeGlib.BoundingBox({ top    : bbox.top,
+            boundingBox: new GeocodeGlib.BoundingBox({ top    : bbox.top,
                                                         bottom : bbox.bottom,
                                                         left   : bbox.left,
                                                         right  : bbox.right })
@@ -830,32 +960,14 @@ export class MapView extends Gtk.Overlay {
         this._turnPointMarker.goTo();
     }
 
+    /**
+     * @param {StoredRoute} stored
+     */
     _showStoredRoute(stored) {
-        let query = Application.routeQuery;
-        let route = Application.routingDelegator.graphHopper.route;
-
-        Application.routingDelegator.graphHopper.storedRoute = stored.route;
-
-        let resetId = route.connect('reset', function() {
-            route.disconnect(resetId);
-            query.freeze_notify();
-
-            let storedLast = stored.places.length - 1;
-            query.points[0].place = stored.places[0];
-            query.points[1].place = stored.places[storedLast];
-            query.transportation = stored.transportation;
-
-            for (let i = 1; i < storedLast; i++) {
-                let point = query.addPoint(i);
-                point.place = stored.places[i];
-            }
-
-            query.thaw_notify();
-        });
-        route.reset();
+        Application.routingDelegator.replaceRoute(stored);
     }
 
-    showPlace(place, animation) {
+    showPlace(place, animation, skipGoTo) {
         this._placeLayer.remove_all();
 
         if (place instanceof StoredRoute) {
@@ -868,7 +980,11 @@ export class MapView extends Gtk.Overlay {
                                             mapView: this });
 
         this._placeLayer.add_marker(placeMarker);
-        placeMarker.goToAndSelect(animation);
+        if (skipGoTo) {
+            placeMarker.showBubble();
+        } else {
+            placeMarker.goToAndSelect(animation);
+        }
         Application.application.selected_place = place;
     }
 
@@ -879,7 +995,8 @@ export class MapView extends Gtk.Overlay {
         this._placeLayer.remove_all();
 
         routeLayer = this._createRouteLayer(false, TURN_BY_TURN_ROUTE_COLOR,
-                                            null, ROUTE_LINE_WIDTH);
+                                            TURN_BY_TURN_ROUTE_OUTLINE_COLOR,
+                                            ROUTE_LINE_WIDTH + 4, 2);
         route.path.forEach((polyline) => routeLayer.add_node(polyline));
         this.routingOpen = true;
 
@@ -888,7 +1005,7 @@ export class MapView extends Gtk.Overlay {
     }
 
     _showDestinationTurnpoints() {
-        let route = Application.routingDelegator.graphHopper.route;
+        let route = Application.routingDelegator.route;
         let query = Application.routeQuery;
         let pointIndex = 0;
 
@@ -1005,7 +1122,15 @@ export class MapView extends Gtk.Overlay {
         this.emit('marker-selected', selectedMarker);
     }
 
-    _onClickGesturePressed(gesture, n_presses, x, y) {
+    _onPrimaryClick(gesture, n_presses, x, y) {
+        if (n_presses > 1)
+            return;
+
+        this.map.grab_focus();
+        gesture.set_state(Gtk.EventSequenceState.NONE);
+    }
+
+    _onSecondaryClick(gesture, n_presses, x, y) {
         if (n_presses > 1) {
             gesture.set_state(Gtk.EventSequenceState.DENIED);
             return;
