@@ -21,27 +21,29 @@
  */
 
 import GObject from 'gi://GObject';
-import Gtk from 'gi://Gtk';
+import Adw from 'gi://Adw';
 
 import {Application} from './application.js';
 import * as Utils from './utils.js';
 
-export class OSMAccountDialog extends Gtk.Dialog {
+export class OSMAccountDialog extends Adw.Window {
 
     static Response = { SIGNED_IN: 0 };
 
     constructor({closeOnSignIn, ...params}) {
-        /* use_header_bar is a construct-only property and cannot be set by GtkBuilder */
-        super({...params, use_header_bar: true});
+        super({...params});
+
+        this.connect('close-request', () => this.emit('response', this.response));
 
         this._closeOnSignIn = closeOnSignIn;
+
         this._signInButton.connect('clicked',
                                    this._onSignInButtonClicked.bind(this));
         this._verifyButton.connect('clicked',
                                    this._onVerifyButtonClicked.bind(this));
         this._verificationEntry.connect('changed',
                                         this._onVerificationEntryChanged.bind(this));
-        this._verificationEntry.connect('activate',
+        this._verificationEntry.connect('entry-activated',
                                         this._onVerificationEntryActivated.bind(this));
         this._signOutButton.connect('clicked',
                                     this._onSignOutButtonClicked.bind(this));
@@ -49,7 +51,7 @@ export class OSMAccountDialog extends Gtk.Dialog {
         /* if the user is logged in, show the logged-in view */
         if (Application.osmEdit.isSignedIn) {
             this._updateSignedInUserLabel();
-            this._stack.visible_child_name = 'logged-in';
+            this._navigationView.replace_with_tags(['logged-in']);
         }
     }
 
@@ -72,7 +74,7 @@ export class OSMAccountDialog extends Gtk.Dialog {
 
     _performSignIn() {
         // switch to the verification view
-        this._stack.visible_child_name = 'verify';
+        this._navigationView.replace_with_tags(['verify']);
 
         Application.osmEdit.performOAuthSignIn();
     }
@@ -117,23 +119,21 @@ export class OSMAccountDialog extends Gtk.Dialog {
             this._updateSignedInUserLabel();
 
             if (this._closeOnSignIn) {
-                this.response(Response.SIGNED_IN);
+                this.response = OSMAccountDialog.Response.SIGNED_IN;
+                this.close();
             } else {
                 /* switch to the logged in view and reset the state in case
                    the user signs out and start over again */
-                this._errorLabel.visible = false;
-                this._stack.visible_child_name = 'logged-in';
+                this._navigationView.replace_with_tags(['logged-in']);
             }
         } else {
             if (errorMessage)
                 Utils.showToastInOverlay(errorMessage, this._overlay);
-            /* switch back to the sign-in view, and show a label indicating
+            /* switch back to the sign-in view, and show a toast indicating
                that verification failed */
-            this._errorLabel.visible = true;
-            this._errorLabel.label =
-                _("The verification code didn’t match, please try again.");
+            Utils.showToastInOverlay(_("The verification code didn’t match, please try again."), this._overlay);
             this._signInButton.sensitive = true;
-            this._stack.visible_child_name = 'sign-in';
+            this._navigationView.replace_with_tags(['sign-in']);
         }
         /* reset verification code entry */
         this._verificationEntry.text = '';
@@ -142,17 +142,25 @@ export class OSMAccountDialog extends Gtk.Dialog {
     _onSignOutButtonClicked() {
         Application.osmEdit.signOut();
         this._signInButton.sensitive= true;
-        this._stack.visible_child_name = 'sign-in';
+        this._navigationView.replace_with_tags(['sign-in']);
     }
 }
 
 GObject.registerClass({
     Template: 'resource:///org/gnome/Maps/ui/osm-account-dialog.ui',
-    InternalChildren: ['stack',
+    Properties: {
+        'response': GObject.ParamSpec.int('response', '', '',
+                                             GObject.ParamFlags.READABLE |
+                                             GObject.ParamFlags.WRITABLE,
+                                             0),
+    },
+    Signals: {
+        'response': { param_types: [GObject.TYPE_INT] },
+    },
+    InternalChildren: ['navigationView',
                        'signInButton',
                        'verificationEntry',
                        'verifyButton',
-                       'errorLabel',
                        'signedInUserLabel',
                        'signOutButton',
                        'overlay'],
