@@ -23,10 +23,11 @@ import gettext from 'gettext';
 
 import GeocodeGlib from 'gi://GeocodeGlib';
 import Gio from 'gi://Gio';
-import GLib from 'gi://GLib';
 import GObject from 'gi://GObject';
 
+import * as Address from './address.js';
 import {Location} from './location.js';
+import * as OSMNames from './osmNames.js';
 import {Overpass} from './overpass.js';
 import * as PlaceIcons from './placeIcons.js';
 import * as URIS from './uris.js';
@@ -46,131 +47,207 @@ const DMS_COORDINATES_REGEX = new RegExp(
     "i"
 );
 
-export class Place extends GeocodeGlib.Place {
+export class Place extends GObject.Object {
 
     constructor(params) {
-        let originalParams = {};
+        super();
 
-        Object.assign(originalParams, params);
+        /* Some of these properties are read as snake_case for backward compatibility
+        with the place store. */
+        this._osmId = params.osmId;
+        this._osmType = params.osmType;
+        this._name = params.name;
+        this._description = params.description;
+        this._source = params.source;
+        this._location = params.location;
+        this._boundingBox = params.boundingBox ?? null;
+        this._placeType = params.placeType;
+        this._streetAddress = params.streetAddress;
+        this._street = params.street;
+        this._building = params.building;
+        this._postalCode = params.postalCode;
+        this._area = params.area;
+        this._town = params.town;
+        this._state = params.state;
+        this._county = params.county;
+        this._country = params.country;
+        this._countryCode = params.countryCode;
+        this._continent = params.continent;
+        this._originalJson = params.originalJson;
 
-        delete params.population;
-        delete params.website;
-        delete params.email;
-        delete params.phone;
-        delete params.wiki;
-        delete params.wikidata;
-        delete params.openingHours;
-        delete params.internetAccess;
-        delete params.religion;
-        delete params.takeaway;
-        delete params.note;
-        delete params.isCurrentLocation;
-        delete params.initialZoom;
-        delete params.prefilled;
-        delete params.store;
-        delete params.wheelchair;
-        delete params.nativeName;
-        delete params.osmKey;
-        delete params.osmValue;
-
-        if (originalParams.place) {
-            params = { osm_id: originalParams.place.osm_id,
-                       osm_type: originalParams.place.osm_type,
-                       name: originalParams.place.name,
-                       location: originalParams.place.location,
-                       bounding_box: originalParams.place.bounding_box,
-                       place_type: originalParams.place.place_type,
-                       street_address: originalParams.place.street_address,
-                       street: originalParams.place.street,
-                       building: originalParams.place.building,
-                       postal_code: originalParams.place.postal_code,
-                       area: originalParams.place.area,
-                       town: originalParams.place.town,
-                       state: originalParams.place.state,
-                       county: originalParams.place.county,
-                       country: originalParams.place.country,
-                       country_code: originalParams.place.country_code,
-                       continent: originalParams.place.continent };
-        }
-
-        for (let prop in params)
-            if (!params[prop])
-                delete params[prop];
-
-        super(params);
-
-        this.updateFromTags(originalParams);
-
-        this._isCurrentLocation = originalParams.isCurrentLocation;
-        this._initialZoom = originalParams.initialZoom;
+        this._isCurrentLocation = params.isCurrentLocation;
+        this._initialZoom = params.initialZoom;
 
         /* set to true if the instance is pre-filled with Overpass data,
          * at the time of creation, as will be the case when loading a place
          * from an OSM object URL
          */
-        this._prefilled = originalParams.prefilled;
+        this._prefilled = params.prefilled;
 
         /* Determines if the place should be added to the place store */
-        if (typeof(originalParams.store) === 'undefined') {
+        if (typeof(params.store) === 'undefined') {
             this._store = true;
         } else {
-            this._store = originalParams.store;
+            this._store = params.store;
         }
 
-        this._nativeName = originalParams.nativeName;
-        this._osmKey = originalParams.osmKey;
-        this._osmValue = originalParams.osmValue;
+        this._nativeName = params.nativeName;
+        this._osmKey = params.osmKey;
+        this._osmValue = params.osmValue;
+        this._osmTags = params.osmTags;
     }
 
-    /**
-     * Update place with values from OSM tags.
-     */
-    updateFromTags(tags) {
-        /* special handle tags where we use a different name compared to
-         * OSM, to remain backwards-compatible with the place store
-         */
-        let wiki = tags.wiki ?? tags.wikipedia;
-        let openingHours = tags.openingHours ?? tags.opening_hours;
-        let internetAccess = tags.internetAccess ?? tags.internet_access;
+    get osmId() {
+        return this._osmId;
+    }
 
-        if (tags.name)
-            this.nativeName = tags.name;
-        if (tags.population)
-            this.population = tags.population;
-        if (tags['contact:website'])
-            this.website = tags['contact:website'];
-        if (tags.website)
-            this.website = tags.website;
-        if (tags['contact:email'])
-            this.email = tags['contact:email'];
-        if (tags.email)
-            this.email = tags.email;
-        if (tags['contact:phone'])
-            this.phone = tags['contact:phone'];
-        if (tags.phone)
-            this.phone = tags.phone;
-        if (wiki)
-            this.wiki = wiki;
-        if (tags.wikidata)
-            this.wikidata = tags.wikidata;
-        if (tags.wheelchair)
-            this.wheelchair = tags.wheelchair;
-        if (openingHours)
-            this.openingHours = openingHours;
-        if (internetAccess)
-            this.internetAccess = internetAccess;
-        if (tags.ele && this.location)
-            this.location.altitude = parseFloat(tags.ele);
-        else if (tags.ele && tags.location)
-            tags.location.altitude = tags.ele;
-        if (tags.religion)
-            this.religion = tags.religion
-        if (tags.toilets)
-            this.toilets = tags.toilets;
-        if (tags.takeaway)
-            this.takeaway = tags.takeaway;
-        if (tags.note)
-            this.note = tags.note;
+    set osmId(osmId) {
+        this._osmId = osmId;
+    }
+
+    get osmType() {
+        return this._osmType;
+    }
+
+    set osmType(osmType) {
+        this._osmType = osmType;
+    }
+
+    get osmTags() {
+        return this._osmTags;
+    }
+
+    set osmTags(osmTags) {
+        this._osmTags = osmTags;
+
+        if (osmTags.ele && this.location)
+            this.location.altitude = parseFloat(osmTags.ele);
+    }
+
+    get name() {
+        if (this.osmTags) {
+            const language = Utils.getLanguage();
+
+            return OSMNames.getNameForLanguageAndCountry(
+                this.osmTags, language, this.countryCode
+            );
+        }
+
+        return this._name;
+    }
+
+    set name(name) {
+        this._name = name;
+    }
+
+    get description() {
+        return this._description;
+    }
+
+    set description(description) {
+        this._description = description;
+    }
+
+    get source() {
+        return this._source;
+    }
+
+    set source(source) {
+        this._source = source;
+    }
+
+    get location() {
+        return this._location;
+    }
+
+    set location(location) {
+        this._location = location;
+    }
+
+    get boundingBox() {
+        return this._boundingBox;
+    }
+
+    set boundingBox(boundingBox) {
+        this._boundingBox = boundingBox;
+    }
+
+    get placeType() {
+        return this._placeType;
+    }
+
+    set placeType(placeType) {
+        this._placeType = placeType;
+    }
+
+    get streetAddress() {
+        if (this.houseNumber && this.street) {
+            return Address.streetAddressForCountryCode(
+                this.street, this.houseNumber, this.countryCode
+            );
+        }
+
+        return this._streetAddress;
+    }
+
+    get street() {
+        return this.osmTags?.['addr:street'] ?? this._street;
+    }
+
+    get houseNumber() {
+        return this.osmTags?.['addr:housenumber'];
+    }
+
+    get building() {
+        return this._building;
+    }
+
+    set building(building) {
+        this._building = building;
+    }
+
+    get postalCode() {
+        return this.osmTags?.['addr:postcode'] ?? this._postalCode;
+    }
+
+    get area() {
+        return this._area;
+    }
+
+    get town() {
+        return this.osmTags?.['addr:city'] ?? this._town;
+    }
+
+    get state() {
+        return this._state;
+    }
+
+    get county() {
+        return this._county;
+    }
+
+    get country() {
+        return this._country;
+    }
+
+    get countryCode() {
+        let result = this.osmTags?.['addr:country'] ?? this._countryCode;
+        if (result)
+            return result;
+
+        this._countryCode = Utils.getCountryCodeForCoordinates(
+            this.location.latitude,
+            this.location.longitude
+        );
+        return this._countryCode;
+    }
+
+    get continent() {
+        return this._continent;
+    }
+
+    set continent(continent) {
+        this._continent = continent;
     }
 
     set store(v) {
@@ -182,115 +259,67 @@ export class Place extends GeocodeGlib.Place {
     }
 
     get uniqueID() {
-        return this.osm_type + '-' + this.osm_id;
+        return this.osmType + '-' + this.osmId;
     }
 
     get isCurrentLocation() {
         return this._isCurrentLocation;
     }
 
-    set population(v) {
-        this._population = v;
-    }
-
     get population() {
-        return this._population;
-    }
-
-    set website(v) {
-        this._website = v;
+        return this.osmTags?.population;
     }
 
     get website() {
-        return this._website;
-    }
-
-    set email(v) {
-        this._email = v;
+        return this.osmTags?.['contact:website'] ?? this.osmTags?.website;
     }
 
     get email() {
-        return this._email;
-    }
-
-    set phone(v) {
-        this._phone = v;
+        return this.osmTags?.['contact:email'] ?? this.osmTags?.email;
     }
 
     get phone() {
-        return this._phone;
-    }
-
-    set wiki(v) {
-        this._wiki = v;
+        return this.osmTags?.['contact:phone'] ?? this.osmTags?.phone;
     }
 
     get wiki() {
-        return this._wiki;
-    }
-
-    set wikidata(v) {
-        this._wikidata = v;
+        return this.osmTags?.wikipedia ?? this.osmTags?.wiki;
     }
 
     get wikidata() {
-        return this._wikidata;
-    }
-
-    set openingHours(v) {
-        this._openingHours = v;
+        return this.osmTags?.wikidata;
     }
 
     get openingHours() {
-        return this._openingHours;
-    }
-
-    set internetAccess(v) {
-        this._internetAccess = v;
+        return this.osmTags?.opening_hours ?? this.osmTags?.openingHours;
     }
 
     get internetAccess() {
-        return this._internetAccess;
-    }
-
-    set religion(v) {
-        this._religion = v;
+        return this.osmTags?.internet_access ?? this.osmTags?.internetAccess;
     }
 
     get religion() {
-        return this._religion;
+        return this.osmTags?.religion;
     }
 
-    set toilets(v) {
-        this._toilets = v;
+    get sport() {
+        return this.osmTags?.sport;
     }
 
     get toilets() {
-        return this._toilets;
-    }
-
-    set takeaway(v) {
-        this._takeaway = v;
+        return this.osmTags?.toilets;
     }
 
     get takeaway() {
-        return this._takeaway;
-    }
-
-    set note(v) {
-        this._note = v;
+        return this.osmTags?.takeaway;
     }
 
     get note() {
-        return this._note;
-    }
-
-    set wheelchair(v) {
-        this._wheelchair = v;
+        return this.osmTags?.note;
     }
 
     get wheelchair() {
-        return this._wheelchair;
+        return this.osmTags?.wheelchair;
     }
 
     get initialZoom() {
@@ -306,18 +335,25 @@ export class Place extends GeocodeGlib.Place {
     }
 
     get nativeName() {
-        return this._nativeName;
-    }
-
-    set nativeName(nativeName) {
-        this._nativeName = nativeName;
+        return this.osmTags?.name;
     }
 
     /**
      * Most important OSM tag for the place ('amenity', 'shop', and so on).
-     * This corresponds to the osm_key parameter in Photon geocoder
      */
     get osmKey() {
+        const osmTags = this.osmTags;
+
+        if (osmTags) {
+            const tag = [
+                'place', 'amenity', 'leisure', 'shop', 'tourism', 'highway',
+                'railway', 'aeroway', 'building'
+            ].find(key => key in osmTags);
+
+            if (tag)
+                return tag;
+        }
+
         return this._osmKey;
     }
 
@@ -326,7 +362,7 @@ export class Place extends GeocodeGlib.Place {
      * This corresponds to the osm_value parameter in Photon geocoder.
      */
     get osmValue() {
-        return this._osmValue;
+        return this.osmTags?.[this.osmKey] ?? this._osmValue;
     }
 
     get icon() {
@@ -342,60 +378,60 @@ export class Place extends GeocodeGlib.Place {
      * such as restaurants, cafes, pubs and similar
      */
     get isEatingAndDrinking() {
-        return this._osmKey === 'amenity' &&
+        return this.osmKey === 'amenity' &&
                ['bar', 'biergarten', 'cafe', 'fast_food', 'food_court',
-                'restaurant', 'pub'].indexOf(this._osmValue) !== -1;
+                'restaurant', 'pub'].indexOf(this.osmValue) !== -1;
+    }
+
+    get level() {
+        return this.osmTags?.level;
+    }
+
+    get floor() {
+        return this.osmTags?.['addr:floor'] ?? this.osmTags?.['level:ref'];
     }
 
     toJSON() {
-        let bounding_box = null;
+        let boundingBox = null;
 
-        if (this.bounding_box) {
-            bounding_box = { top: this.bounding_box.top,
-                             bottom: this.bounding_box.bottom,
-                             left: this.bounding_box.left,
-                             right: this.bounding_box.right };
+        if (this.boundingBox) {
+            boundingBox = { top: this.boundingBox.top,
+                             bottom: this.boundingBox.bottom,
+                             left: this.boundingBox.left,
+                             right: this.boundingBox.right };
         }
 
-        let location = { latitude: this.location.latitude,
-                         longitude: this.location.longitude,
-                         altitude: this.location.altitude,
-                         accuracy: this.location.accuracy };
+        let location = {
+            latitude: this.location.latitude,
+            longitude: this.location.longitude,
+            altitude: this.location.altitude > -1000000 ? this.location.altitude : undefined,
+            accuracy: this.location.accuracy >= 0 ? this.location.accuracy : undefined,
+        };
 
-        return { id: this.osm_id,
-                 osm_type: this.osm_type,
+        return { ...this._originalJson,
+                 osmId: this._osmId,
+                 osmType: this._osmType,
                  osmKey: this._osmKey,
                  osmValue: this._osmValue,
-                 place_type: this.place_type,
-                 name: this.name,
-                 nativeName: this.nativeName,
-                 bounding_box: bounding_box,
-                 this_type: this.this_type,
+                 osmTags: this._osmTags,
+                 placeType: this._placeType,
+                 name: this._name,
+                 description: this._description,
+                 source: this._source,
+                 nativeName: this._nativeName,
+                 boundingBox,
                  location: location,
-                 street_address: this.street_address,
-                 street: this.street,
-                 building: this.building,
-                 postal_code: this.postal_code,
-                 area: this.area,
-                 town: this.town,
-                 state: this.state,
-                 county: this.county,
-                 country: this.country,
-                 country_code: this.country_code,
-                 continent: this.continent,
-                 population: this.population,
-                 website: this.website,
-                 email: this.email,
-                 phone: this.phone,
-                 wiki: this.wiki,
-                 wikidata: this.wikidata,
-                 wheelchair: this.wheelchair,
-                 openingHours: this.openingHours,
-                 internetAccess: this.internetAccess,
-                 religion: this.religion,
-                 toilets: this.toilets,
-                 takeaway: this.takeaway,
-                 note: this.note };
+                 streetAddress: this._streetAddress,
+                 street: this._street,
+                 building: this._building,
+                 postalCode: this._postalCode,
+                 area: this._area,
+                 town: this._town,
+                 state: this._state,
+                 county: this._county,
+                 country: this._country,
+                 countryCode: this._countryCode,
+                 continent: this._continent };
     }
 
     match(searchString) {
@@ -430,17 +466,13 @@ export class Place extends GeocodeGlib.Place {
             let prop = obj[key];
 
             switch(key) {
-                case 'id':
-                    props.osm_id = prop;
-                    break;
-
                 case 'location':
                     props.location = new Location(prop);
                     break;
 
-                case 'bounding_box':
+                case 'boundingBox':
                     if (prop)
-                        props.bounding_box = new GeocodeGlib.BoundingBox(prop);
+                        props.boundingBox = new GeocodeGlib.BoundingBox(prop);
                     break;
 
                 default:
@@ -449,6 +481,7 @@ export class Place extends GeocodeGlib.Place {
                     break;
             }
         }
+        props.originalJson = obj;
         return new Place(props);
     }
 
@@ -524,10 +557,10 @@ function _parseHttpURL(text, callback) {
     let [type, id] = URIS.parseAsObjectURL(text);
 
     if (type && id) {
-        let storedPlace = Application.placeStore.existsWithOsmTypeAndId(type, id);
+        let storedPlace = Application.placeStore.getByOsmId(Utils.osmTypeFromString(type), id);
 
         if (storedPlace) {
-            callback(storedPlace, null);
+            callback(storedPlace.place, null);
             return;
         }
 
