@@ -23,7 +23,6 @@
 import gettext from 'gettext';
 
 import Gdk from 'gi://Gdk';
-import GLib from 'gi://GLib';
 import GObject from 'gi://GObject';
 import GeocodeGlib from 'gi://GeocodeGlib';
 import Gio from 'gi://Gio';
@@ -31,12 +30,11 @@ import Gtk from 'gi://Gtk';
 
 import {Application} from './application.js';
 import * as GeocodeFactory from './geocode.js';
-import {Location} from './location.js';
 import {Place} from './place.js';
 import {PlaceStore} from './placeStore.js';
 import {PlacePopover} from './placePopover.js';
 import * as URIS from './uris.js';
-import * as Utils from './utils.js';
+import { StoredRoute } from './storedRoute.js';
 
 const _ = gettext.gettext;
 
@@ -85,13 +83,20 @@ export class PlaceEntry extends Gtk.SearchEntry {
         return this._popover;
     }
 
-    constructor({mapView, maxChars, matchRoute, ...params}) {
+    get mapView() {
+        return this._mapView;
+    }
+
+    constructor({mapView, maxChars, matchRoute, hasPoiBrowser = false,
+                 popoverParent, ...params}) {
         super(params);
 
         this._mapView = mapView;
         this._matchRoute = matchRoute ?? false;
-        this._filter = new Gtk.TreeModelFilter({ child_model: Application.placeStore });
-        this._filter.set_visible_func(this._completionVisibleFunc.bind(this));
+        this._filter = new Gtk.FilterListModel({
+            model: Application.placeStore,
+            filter: Gtk.CustomFilter.new(this._completionVisibleFunc.bind(this)),
+        });
 
         this._keyController = new Gtk.EventControllerKey();
         this._keyController.connect('key-pressed', this._onKeyPressed.bind(this));
@@ -99,7 +104,8 @@ export class PlaceEntry extends Gtk.SearchEntry {
 
         this.connect('activate', this._onActivate.bind(this));
 
-        this._popover = this._createPopover(maxChars);
+        this._popover =
+            this._createPopover(maxChars, hasPoiBrowser, popoverParent ?? this);
 
         this.connect('search-changed', this._onSearchChanged.bind(this));
 
@@ -135,22 +141,14 @@ export class PlaceEntry extends Gtk.SearchEntry {
         if ((this.text.length >= MIN_CHARS_COMPLETION ||
              (this.text.length > 0 && this.text[0].match(IDEOGRAPH_PATTERN))) &&
             this.text !== this._placeText) {
-            let cachedResults = this._cache[this.text];
-
-            if (cachedResults) {
-                this.updateResults(cachedResults, this.text, true);
-            } else {
-                // if no previous search has been performed, show spinner
-                if (!this._previousSearch ||
-                    this._previousSearch.length < MIN_CHARS_COMPLETION ||
-                    this._placeText) {
-                    this._popover.showSpinner();
-                }
-                this._placeText = '';
-                this._doSearch();
-            }
+            this._handleSearchInput();
         } else {
-            this._popover.popdown();
+            /* if the popover is showing the POI browser, don't hide it,
+             * this prevents the popover closing and popping up again if
+             * the user starts typing to do a free-text search
+             */
+            if (!this._popover.isShowingPoiBrowser)
+                this._popover.popdown();
             this.grab_focus();
             if (this.text.length === 0)
                 this.place = null;
@@ -158,6 +156,23 @@ export class PlaceEntry extends Gtk.SearchEntry {
         }
     }
 
+    _handleSearchInput() {
+        let cachedResults = this._cache[this.text];
+
+        if (cachedResults) {
+            this.updateResults(cachedResults, this.text, true);
+        } else {
+            // if no previous search has been performed, show spinner
+            if (!this._previousSearch ||
+                this._previousSearch.length < MIN_CHARS_COMPLETION ||
+                this._placeText) {
+                this._popover.showSpinner();
+            }
+            this._placeText = '';
+            this._doSearch();
+        }
+    }
+
     _locEquals(placeA, placeB) {
         if (!placeA.location || !placeB.location)
             return false;
@@ -166,11 +181,11 @@ export class PlaceEntry extends Gtk.SearchEntry {
                 placeA.location.longitude === placeB.location.longitude);
     }
 
-    _createPopover(maxChars) {
+    _createPopover(maxChars, hasPoiBrowser, popoverParent) {
         let popover = new PlacePopover({ entry:         this,
-                                         maxChars:      maxChars });
-
-        popover.set_parent(this);
+                                         maxChars:      maxChars,
+                                         hasPoiBrowser: hasPoiBrowser });
+        popover.set_parent(popoverParent);
         this.set_key_capture_widget(popover);
 
         popover.connect('selected', (widget, place) => {
@@ -202,20 +217,7 @@ export class PlaceEntry extends Gtk.SearchEntry {
                 return true;
             }
 
-            let row = this._popover.list.get_selected_row();
-            let idx;
-
-            if (!row)
-                idx = (direction === 1) ? 0 : length - 1;
-            else
-                idx = row.get_index() + direction;
-
-            let inBounds = 0 <= idx && idx < length;
-
-            if (inBounds)
-                this._popover.selectRow(this._popover.list.get_row_at_index(idx));
-            else
-                this._popover.list.unselect_all();
+            this._popover.handleArrowKey(direction);
 
             return true;
         }
@@ -224,23 +226,28 @@ export class PlaceEntry extends Gtk.SearchEntry {
     }
 
     _onActivate() {
-        // if the popover is visible and we have a selected item, activate it
-        let row = this._popover.list.get_selected_row();
-
-        if (this._popover.visible && row)
-            row.activate();
+        /* if the search result is short enough and doesn't start with a CJK
+         * ideograph, it wouldn't already have been triggered in the "search
+         * changed" handler.
+         * In case the popover isn't already showing (e.g. showing POI search)
+         * trigger a search, allowing explicitly searching for short strings
+         */
+        if (!this._popover.visible && this.text.length > 0 &&
+            this.text.length < MIN_CHARS_COMPLETION &&
+            !(this.text.length > 0 && this.text[0].match(IDEOGRAPH_PATTERN))) {
+            this._handleSearchInput();
+        } else {
+            this._popover.handleActivate();
+        }
     }
 
-    _completionVisibleFunc(model, iter) {
-        let place = model.get_value(iter, PlaceStore.Columns.PLACE);
-        let type = model.get_value(iter, PlaceStore.Columns.TYPE);
-
-        if (type !== PlaceStore.PlaceType.RECENT_ROUTE ||
-            (!this._matchRoute && type === PlaceStore.PlaceType.RECENT_ROUTE))
+    _completionVisibleFunc(placeItem) {
+        if (!(placeItem.place instanceof StoredRoute) ||
+            (!this._matchRoute && placeItem.type instanceof StoredRoute))
             return false;
 
-        if (place !== null)
-            return place.match(this.text);
+        if (placeItem.place !== null)
+            return placeItem.place.match(this.text);
         else
             return false;
     }
@@ -264,8 +271,7 @@ export class PlaceEntry extends Gtk.SearchEntry {
                 location.set_from_uri(this.text);
                 this.place = new Place({ location: location });
             } catch(e) {
-                let msg = _("Failed to parse Geo URI");
-                Utils.showDialog(msg, Gtk.MessageType.ERROR, this.get_toplevel());
+                this.root.showToast(_("Failed to parse Geo URI"));
             }
 
             parsed = true;
@@ -277,8 +283,7 @@ export class PlaceEntry extends Gtk.SearchEntry {
             if (query) {
                 this.text = query;
             } else {
-                let msg = _("Failed to parse Maps URI");
-                Utils.showDialog(msg, Gtk.MessageType.ERROR, this.get_toplevel());
+                this.root.showToast(_("Failed to parse Maps URI"));
             }
 
             parsed = true;
@@ -306,8 +311,7 @@ export class PlaceEntry extends Gtk.SearchEntry {
                 if (place)
                     this.place = place;
                 else
-                    Utils.showDialog(error,
-                                     Gtk.MessageType.ERROR, this.get_toplevel());
+                    this.root.showToast(error);
             });
 
             /* don't cancel ongoing search, as we have started an async
@@ -353,9 +357,9 @@ export class PlaceEntry extends Gtk.SearchEntry {
 
     /**
      * Update results popover
-     * places array of places from search result
-     * searchText original search string to highlight in results
-     * includeContactsAndRoute whether to include contacts and recent routes
+     * @param {Place[]} places array of places from search result
+     * @param {string} searchText original search string to highlight in results
+     * @param {boolean} includeContactsAndRoute whether to include contacts and recent routes
      *                         among results
      */
     updateResults(places, searchText, includeContactsAndRoutes) {
@@ -367,35 +371,29 @@ export class PlaceEntry extends Gtk.SearchEntry {
 
         let completedPlaces = [];
 
-
         if (includeContactsAndRoutes) {
-            this._filter.refilter();
-            this._filter.foreach((model, path, iter) => {
-                let place = model.get_value(iter, PlaceStore.Columns.PLACE);
-                let type = model.get_value(iter, PlaceStore.Columns.TYPE);
-
-                completedPlaces.push({ place: place, type: type });
-            });
+            this._filter.filter.changed(Gtk.FilterChange.DIFFERENT);
+            for (let i = 0; i < this._filter.n_items; i++) {
+                const placeItem = this._filter.get_item(i);
+                completedPlaces.push(placeItem.place);
+            }
         }
 
-        let placeStore = Application.placeStore;
-
-        places.forEach((place) => {
-            let type;
-
-            if (placeStore.exists(place, PlaceStore.PlaceType.RECENT))
-                type = PlaceStore.PlaceType.RECENT;
-            else if (placeStore.exists(place, PlaceStore.PlaceType.FAVORITE))
-                type = PlaceStore.PlaceType.FAVORITE;
-            else
-                type = PlaceStore.PlaceType.ANY;
-
-            completedPlaces.push({ place: place, type: type });
-        });
+        completedPlaces = completedPlaces.concat(places);
 
         this._popover.updateResult(completedPlaces, searchText);
         this._popover.showResult();
     }
+
+    browsePois() {
+        this._popover.showPoiMainCategories();
+        /* clear text entry incase it has a previous selected place
+         * highlighted to avoid confusion, as this text is not taken into
+         * account when searching for POIs
+         */
+        this.text = '';
+        this.grab_focus();
+    }
 }
 
 GObject.registerClass({
@@ -405,6 +403,6 @@ GObject.registerClass({
                                           'The selected place',
                                           GObject.ParamFlags.READABLE |
                                           GObject.ParamFlags.WRITABLE,
-                                          GeocodeGlib.Place)
+                                          Place)
     }
 }, PlaceEntry);
