@@ -24,8 +24,8 @@ import GObject from 'gi://GObject';
 import Gtk from 'gi://Gtk';
 
 import {PlaceFormatter} from './placeFormatter.js';
-import {PlaceStore} from './placeStore.js';
 import * as Utils from './utils.js';
+import { Application } from './application.js';
 
 const C_ = gettext.dgettext;
 
@@ -47,15 +47,21 @@ const SHORT_DISTANCE_FORMAT = C_("short distance format string", "< %s");
 
 export class PlaceListRow extends Gtk.ListBoxRow {
 
-    constructor({place, searchString, type, sizeGroup, ...params}) {
+    constructor({place, searchString, sizeGroup, showSecondaryIcon, ...params}) {
         super(params);
 
-        this.update(place, type, searchString || '');
+        this.update(place, searchString || '');
         if (sizeGroup)
             sizeGroup.add_widget(this._distanceLabel);
+
+        this._showSecondaryIcon = showSecondaryIcon ?? true;
     }
 
-    update(place, type, searchString) {
+    /**
+     * @param {Place} place
+     * @param {string} searchString
+     */
+    update(place, searchString) {
         this.place = place;
         let formatter = new PlaceFormatter(this.place);
         let markup = GLib.markup_escape_text(formatter.title, -1);
@@ -67,13 +73,15 @@ export class PlaceListRow extends Gtk.ListBoxRow {
         if (place.icon)
             this._icon.gicon = place.icon;
 
-        if (type === PlaceStore.PlaceType.RECENT ||
-            type === PlaceStore.PlaceType.RECENT_ROUTE)
-            this._typeIcon.icon_name = 'document-open-recent-symbolic';
-        else if (type === PlaceStore.PlaceType.FAVORITE)
-            this._typeIcon.icon_name = 'starred-symbolic';
-        else
-            this._typeIcon.icon_name = null;
+        const placeItem = Application.placeStore.getPlaceItem(place);
+        if (placeItem && this._showSecondaryIcon) {
+            this._typeIcon.show();
+            this._typeIcon.icon_name = placeItem.isFavorite
+                ? 'starred-symbolic'
+                : 'document-open-recent-symbolic';
+        } else {
+            this._typeIcon.hide();
+        }
 
         /* hide distance by default so that a previous content from a POI
          * search doesn't keep the distance when updating with a new search
