@@ -25,14 +25,12 @@ import Gtk from 'gi://Gtk';
 import {Application} from './application.js';
 import {Overpass} from './overpass.js';
 import {PlaceListRow} from './placeListRow.js';
-import {PlaceStore} from './placeStore.js';
+import {PlaceStoreItem} from './placeStore.js';
 import * as PoiCategories from './poiCategories.js';
 import {PoiCategoryGobackRow} from './poiCategoryGobackRow.js';
 import {PoiCategoryRow} from './poiCategoryRow.js';
 import {SearchPopover} from './searchPopover.js';
 
-const _PLACE_ICON_SIZE = 20;
-
 export class PlacePopover extends SearchPopover {
 
     constructor({maxChars, hasPoiBrowser = false, ...params}) {
@@ -155,10 +153,7 @@ export class PlacePopover extends SearchPopover {
             } else if (results.length === 0) {
                 this.showNoResult();
             } else {
-                let placeStore = Application.placeStore;
-                let completedPlaces = placeStore.getCompletedPlaces(results);
-
-                this.updateResult(completedPlaces, '');
+                this.updateResult(results, '');
                 this._updateDistances();
                 this.showResult();
                 this._entry.grab_focus();
@@ -286,9 +281,9 @@ export class PlacePopover extends SearchPopover {
 
             // update existing row, if there is one, otherwise create new
             if (row)
-                row.update(p.place, p.type, searchString);
+                row.update(p, searchString);
             else
-                this._addRow(p.place, p.type, searchString);
+                this._addRow(p, searchString);
 
             i++;
         });
@@ -314,10 +309,13 @@ export class PlacePopover extends SearchPopover {
         }
     }
 
-    _addRow(place, type, searchString) {
-        let row = new PlaceListRow({ place:        place,
+    /**
+     * @param {Place} placeItem
+     * @param {string} searchString
+     */
+    _addRow(place, searchString) {
+        let row = new PlaceListRow({ place,
                                      searchString: searchString,
-                                     type:         type,
                                      sizeGroup:    this._distanceLabelSizeGroup,
                                      can_focus:    true });
         this.list.insert(row, -1);
