@@ -22,6 +22,8 @@
 import {GraphHopper} from './graphHopper.js';
 import {TransitRouter} from './transitRouter.js';
 import {RouteQuery} from './routeQuery.js';
+import { Route } from './route.js';
+import { StoredRoute } from './storedRoute.js';
 
 const _FALLBACK_TRANSPORTATION = RouteQuery.Transportation.PEDESTRIAN;
 
@@ -29,9 +31,10 @@ export class RoutingDelegator {
 
     constructor({query}) {
         this._query = query;
+        this._route = new Route();
 
         this._transitRouting = false;
-        this._graphHopper = new GraphHopper({ query: this._query });
+        this._graphHopper = new GraphHopper({ query: this._query, route: this._route });
         this._transitRouter = new TransitRouter({ query: this._query });
         this._query.connect('notify::points', this._onQueryChanged.bind(this));
 
@@ -42,6 +45,8 @@ export class RoutingDelegator {
             !this._transitRouter.enabled) {
             this._query.transportation = _FALLBACK_TRANSPORTATION;
         }
+
+        this._ignoreNextQueryChange = false;
     }
 
     get graphHopper() {
@@ -52,6 +57,10 @@ export class RoutingDelegator {
         return this._transitRouter;
     }
 
+    get route() {
+        return this._route;
+    }
+
     set useTransit(useTransit) {
         this._transitRouting = useTransit;
     }
@@ -60,10 +69,15 @@ export class RoutingDelegator {
         if (this._transitRouting)
             this._transitRouter.plan.reset();
         else
-            this._graphHopper.route.reset();
+            this._route.reset();
     }
 
     _onQueryChanged() {
+        if (this._ignoreNextQueryChange) {
+            this._ignoreNextQueryChange = false;
+            return;
+        }
+
         if (this._query.isValid()) {
             this._query.emit('run');
             if (this._transitRouting) {
@@ -74,4 +88,32 @@ export class RoutingDelegator {
             }
         }
     }
+
+    /**
+     * @param {StoredRoute} stored
+     */
+    replaceRoute(stored) {
+        const query = this._query;
+        query.freeze_notify();
+
+        let storedLast = stored.places.length - 1;
+        query.points[0].place = stored.places[0];
+        query.points[1].place = stored.places[storedLast];
+        query.transportation = stored.transportation;
+
+        for (let i = 1; i < storedLast; i++) {
+            let point = query.addPoint(i);
+            point.place = stored.places[i];
+        }
+
+        this._ignoreNextQueryChange = true;
+        query.thaw_notify();
+
+        this.route.update({ path: stored.route.path,
+                            turnPoints: stored.route.turnPoints,
+                            distance: stored.route.distance,
+                            time: stored.route.time,
+                            bbox: stored.route.bbox});
+
+    }
 }
