@@ -234,7 +234,7 @@ export class Sidebar extends Gtk.Grid {
     }
 
     _initInstructionList() {
-        let route = Application.routingDelegator.graphHopper.route;
+        let route = Application.routingDelegator.route;
         let transitPlan = Application.routingDelegator.transitRouter.plan;
 
         route.connect('reset', () => {
@@ -312,16 +312,15 @@ export class Sidebar extends Gtk.Grid {
                     geoclue: Application.geoclue
                 });
 
-                if (!storedRoute.containsNull) {
-                    placeStore.addPlace(storedRoute,
-                                        PlaceStore.PlaceType.RECENT_ROUTE);
+                if (!storedRoute.containsNull && !storedRoute.containsCurrentLocation) {
+                    placeStore.addPlace(storedRoute);
                 }
                 this._storeRouteTimeoutId = 0;
             });
 
             route.turnPoints.forEach((turnPoint) => {
-                let row = new InstructionRow({ visible: true,
-                                               turnPoint: turnPoint });
+                let row = new InstructionRow({ turnPoint: turnPoint,
+                                               transportation: this._query.transportation} );
                 this._instructionList.insert(row, -1);
             });
 
