@@ -30,9 +30,11 @@ import * as Utils from './utils.js';
 import {GoMetro} from './transitplugins/goMetro.js';
 import {OpendataCH} from './transitplugins/opendataCH.js';
 import {OpenTripPlanner} from './transitplugins/openTripPlanner.js';
+import {OpenTripPlanner2} from './transitplugins/openTripPlanner2.js';
 import {Resrobot} from './transitplugins/resrobot.js';
 
-const ALL_PLUGINS = ["GoMetro", "OpendataCH", "OpenTripPlanner", "Resrobot"];
+const ALL_PLUGINS =
+    ["GoMetro", "OpendataCH", "OpenTripPlanner", "OpenTripPlanner2", "Resrobot"];
 
 /**
  * Class responsible for delegating requests to perform routing in transit
@@ -68,7 +70,7 @@ export class TransitRouter {
             // override plugin was specified, try instanciating if not done yet
             if (!this._currPluginInstance) {
                 try {
-                    this._currentPluginInstance = this._instantiatePlugin(pluginOverride);
+                    this._currPluginInstance = this._instantiatePlugin(pluginOverride);
                 } catch (e) {
                     Utils.debug('Unable to instanciate plugin: ' + pluginOverride);
                     throw e;
@@ -123,7 +125,7 @@ export class TransitRouter {
 
     _getMatchingProvidersForPlace(place) {
         let location = place.location;
-        let country = place.country_code ??
+        let country = place.countryCode ??
                       Utils.getCountryCodeForCoordinates(location.latitude,
                                                          location.longitude);
 
